@@ -78,14 +78,15 @@ class FFSimmer:
         self.rff = rff
         self.origin = copy.copy(rff.origin)
 
-    def compute_grid(self, grid, gmpe, myimts):
+    def compute_grid(self, grid, gmpes, myimts):
         """
         Args:
             grid (Grid2D object):
                 Basically the vs30grid that we're mapping, used for its dimensions.
                 The site corrections have to be done elsewhere.
-            gmpe (MultiGMPE object):
-                The MultiGMPE to be used for the simulations.
+            gmpes (dictionary):
+                A dictionary of MultiGMPEs to be used for the simulations keyed to
+                myimts.
             myimts (list):
                 List of OQ IMTs to generate grids and transects for
 
@@ -109,7 +110,7 @@ class FFSimmer:
         olon = hypo.x
         olat = hypo.y
 
-        if len(gmpe[myimts[0]].DEFINED_FOR_STANDARD_DEVIATION_TYPES) > 1:
+        if len(gmpes[myimts[0]].DEFINED_FOR_STANDARD_DEVIATION_TYPES) > 1:
             sd_types = [
                 oqconst.StdDev.TOTAL,
                 oqconst.StdDev.INTER_EVENT,
@@ -120,7 +121,6 @@ class FFSimmer:
                 oqconst.StdDev.TOTAL,
             ]
 
-        #
         gd = grid.getGeoDict()
         # Increase the grid resolution if it is greater the 30 arc-seconds
         delta_y = gd.dy
@@ -172,7 +172,7 @@ class FFSimmer:
 
         for idx in range(nsim):
             rupt = self.rff.retrieve_rupt(idx)
-            dist = Distance.fromSites(gmpe[myimts[0]], sites, rupt)
+            dist = Distance.fromSites(gmpes[myimts[0]], sites, rupt)
             dctx = dist.getDistanceContext()
             for k in dctx.__dict__:
                 if k == "_slots_":
@@ -180,17 +180,17 @@ class FFSimmer:
                 if not isinstance(dctx.__dict__[k], np.ndarray):
                     continue
                 dctx.__dict__[k] = dctx.__dict__[k].flatten()
-            rctx = rupt.getRuptureContext(gmpe[myimts[0]])
+            rctx = rupt.getRuptureContext(gmpes[myimts[0]])
 
             for iimt, simt in enumerate(myimts):
-                mean, stddevs = gmpe[simt].get_mean_and_stddevs(
+                mean, stddevs = gmpes[simt].get_mean_and_stddevs(
                     sctx, rctx, dctx, simt, sd_types
                 )
                 amps[iimt, idx, :] = mean[0]
-                sigs[iimt, idx, :] = stddevs[0][0]
+                sigs[iimt, idx, :] = stddevs[0]
                 if len(sd_types) > 1:
-                    taus[iimt, idx, :] = stddevs[1][0]
-                    phis[iimt, idx, :] = stddevs[2][0]
+                    taus[iimt, idx, :] = stddevs[1]
+                    phis[iimt, idx, :] = stddevs[2]
 
         slat = np.linspace(sgd.ymin, sgd.ymax, sgd.ny)
         repi = prup.computeRepi(
@@ -263,7 +263,7 @@ class FFSimmer:
 
         return griddict
 
-    def compute_points(self, sctx, gmpe, myimts, rock_vs30=760.0, soil_vs30=180.0):
+    def compute_points(self, sctx, gmpes, myimts, rock_vs30=760.0, soil_vs30=180.0):
         """
         Compute median (or mean) ground motions for a set of sites defined in "sctx"
         for the imts in the list "myimts".
@@ -271,8 +271,9 @@ class FFSimmer:
         Args:
             sctx: (Sites object)
                 A sites context object set up for the points being output
-            gmpe: (MultiGMPE object)
-                The gmpe to perform the ground-motion realizations.
+            gmpes: (dictionary)
+                A dictionary (keyed to myimts) of gmpes to perform the ground-motion
+                  realizations.
             myimts (list)
                 List of OQ IMTs to generate rock and site amplified values for
 
@@ -295,7 +296,7 @@ class FFSimmer:
         """
         outdict = {}
 
-        if len(gmpe.DEFINED_FOR_STANDARD_DEVIATION_TYPES) > 1:
+        if len(gmpes[myimts[0]].DEFINED_FOR_STANDARD_DEVIATION_TYPES) > 1:
             sd_types = [
                 oqconst.StdDev.TOTAL,
                 oqconst.StdDev.INTER_EVENT,
@@ -319,7 +320,7 @@ class FFSimmer:
             sx.__dict__[k] = np.hstack(
                 (sctx.__dict__[k], sctx_rock.__dict__[k], sctx_soil.__dict__[k])
             )
-        sx.sids = np.array(list(range(len(sx.lons))))
+        sx.sids = np.array(list(range(np.size(sx.lons))))
         depths = np.zeros_like(sx.lons)
 
         nsd = len(sd_types)
@@ -332,47 +333,55 @@ class FFSimmer:
 
         amps = np.empty((nctx, nimt, nsim, nporig))
         sigs = np.empty((nctx, nimt, nsim, nporig))
-        if len(sd_types) > 1:
+        if nsd > 1:
             taus = np.empty((nctx, nimt, nsim, nporig))
             phis = np.empty((nctx, nimt, nsim, nporig))
 
         for idx in range(nsim):
             rupt = self.rff.retrieve_rupt(idx)
-            dist_obj = Distance(gmpe, sx.lons, sx.lats, depths, rupt)
+            dist_obj = Distance(
+                gmpes[myimts[0]],
+                sx.lons.flatten(),
+                sx.lats.flatten(),
+                depths.flatten(),
+                rupt,
+            )
             dctx = dist_obj.getDistanceContext()
-            rctx = rupt.getRuptureContext(gmpe)
+            rctx = rupt.getRuptureContext(gmpes[myimts[0]])
 
-            mean, stddevs = gmpe.get_mean_and_stddevs(sx, rctx, dctx, myimts, sd_types)
-            for iimt, _ in enumerate(myimts):
+            for ix, oqimt in enumerate(myimts):
+                mean, stddevs = gmpes[oqimt].get_mean_and_stddevs(
+                    sx, rctx, dctx, [oqimt], sd_types
+                )
                 for ictx in range(nctx):
-                    amps[ictx, iimt, idx, :] = mean[iimt][
+                    amps[ictx, ix, idx, :] = mean[0][
                         ictx * nporig : (ictx + 1) * nporig
                     ]
-                    sigs[ictx, iimt, idx, :] = stddevs[iimt * nsd][
+                    sigs[ictx, ix, idx, :] = stddevs[0][
                         ictx * nporig : (ictx + 1) * nporig
                     ]
-                    if len(sd_types) > 1:
-                        taus[ictx, iimt, idx, :] = stddevs[iimt * nsd + 1][
+                    if nsd > 1:
+                        taus[ictx, ix, idx, :] = stddevs[1][
                             ictx * nporig : (ictx + 1) * nporig
                         ]
-                        phis[ictx, iimt, idx, :] = stddevs[iimt * nsd + 2][
+                        phis[ictx, ix, idx, :] = stddevs[2][
                             ictx * nporig : (ictx + 1) * nporig
                         ]
 
         # Compute Mean, Sigma, Phi, and Tau for each IMT
-        for iimt in range(nimt):
-            outdict[myimts[iimt]] = {}
+        for ix in range(nimt):
+            outdict[myimts[ix]] = {}
             for ictx, ctx_name in enumerate(ctxt_names):
                 # Take the mean over the realizations
                 if self.measure == "median":
-                    mean_amps = np.median(amps[ictx, iimt], axis=0)
+                    mean_amps = np.median(amps[ictx, ix], axis=0)
                 else:
-                    mean_amps = np.mean(amps[ictx, iimt], axis=0)
+                    mean_amps = np.mean(amps[ictx, ix], axis=0)
                 # Compute the residuals for the simulations
-                residuals = amps[ictx, iimt] - mean_amps
+                residuals = amps[ictx, ix] - mean_amps
                 # Find the variance over the realizations, this is the variance we
                 # have to account for
-                extra_vars = np.var(amps[ictx, iimt], axis=0)
+                extra_vars = np.var(amps[ictx, ix], axis=0)
 
                 try:
                     avg_extra_tau = np.var(np.mean(residuals, axis=1))
@@ -399,16 +408,16 @@ class FFSimmer:
                 extra_taus = avg_tau_wgt * extra_vars
                 extra_phis = avg_phi_wgt * extra_vars
 
-                mean_sigs = np.sqrt(np.mean(sigs[ictx, iimt] ** 2, axis=0) + extra_vars)
+                mean_sigs = np.sqrt(np.mean(sigs[ictx, ix] ** 2, axis=0) + extra_vars)
                 if len(sd_types) > 1:
                     mean_taus = np.sqrt(
-                        np.mean(taus[ictx, iimt] ** 2, axis=0) + extra_taus
+                        np.mean(taus[ictx, ix] ** 2, axis=0) + extra_taus
                     )
                     mean_phis = np.sqrt(
-                        np.mean(phis[ictx, iimt] ** 2, axis=0) + extra_phis
+                        np.mean(phis[ictx, ix] ** 2, axis=0) + extra_phis
                     )
 
-                outdict[myimts[iimt]].update(
+                outdict[myimts[ix]].update(
                     {
                         ctx_name: {
                             "mean": mean_amps.reshape(sctx.lons.shape),
@@ -417,22 +426,23 @@ class FFSimmer:
                     }
                 )
                 if len(sd_types) > 1:
-                    outdict[myimts[iimt]][ctx_name][
+                    outdict[myimts[ix]][ctx_name][
                         oqconst.StdDev.INTER_EVENT
                     ] = mean_taus.reshape(sctx.lons.shape)
-                    outdict[myimts[iimt]][ctx_name][
+                    outdict[myimts[ix]][ctx_name][
                         oqconst.StdDev.INTRA_EVENT
                     ] = mean_phis.reshape(sctx.lons.shape)
         return outdict
 
-    def true_grid(self, grid, gmpe, myimts):
+    def true_grid(self, grid, gmpes, myimts):
         """
         Perform the simulations on a full grid, including the actual Vs30
         Args:
             grid (Grid2D object):
                 The vs30grid that we're mapping.
-            gmpe (MultiGMPE object):
-                The MultiGMPE to be used for the simulations.
+            gmpes (dictionary):
+                A dictionary of MultiGMPEs to be used for the simulations keyed to
+                myimts.
             myimts (list):
                 List of OQ IMTs to generate grids and transects for
 
@@ -450,7 +460,7 @@ class FFSimmer:
                 }
         """
         griddict = {}
-        if len(gmpe[myimts[0]].DEFINED_FOR_STANDARD_DEVIATION_TYPES) > 1:
+        if len(gmpes[myimts[0]].DEFINED_FOR_STANDARD_DEVIATION_TYPES) > 1:
             sd_types = [
                 oqconst.StdDev.TOTAL,
                 oqconst.StdDev.INTER_EVENT,
@@ -478,7 +488,7 @@ class FFSimmer:
 
         for idx in range(nsim):
             rupt = self.rff.retrieve_rupt(idx)
-            dist = Distance.fromSites(gmpe[myimts[0]], sites, rupt)
+            dist = Distance.fromSites(gmpes[myimts[0]], sites, rupt)
             dctx = dist.getDistanceContext()
             for k in dctx.__dict__:
                 if k == "_slots_":
@@ -486,17 +496,17 @@ class FFSimmer:
                 if not isinstance(dctx.__dict__[k], np.ndarray):
                     continue
                 dctx.__dict__[k] = dctx.__dict__[k].flatten()
-            rctx = rupt.getRuptureContext(gmpe[myimts[0]])
+            rctx = rupt.getRuptureContext(gmpes[myimts[0]])
 
             for iimt, simt in enumerate(myimts):
-                mean, stddevs = gmpe[simt].get_mean_and_stddevs(
+                mean, stddevs = gmpes[simt].get_mean_and_stddevs(
                     sctx, rctx, dctx, simt, sd_types
                 )
                 amps[iimt, idx, :] = mean[0]
-                sigs[iimt, idx, :] = stddevs[0][0]
+                sigs[iimt, idx, :] = stddevs[0]
                 if len(sd_types) > 1:
-                    taus[iimt, idx, :] = stddevs[1][0]
-                    phis[iimt, idx, :] = stddevs[2][0]
+                    taus[iimt, idx, :] = stddevs[1]
+                    phis[iimt, idx, :] = stddevs[2]
 
         # Compute Mean, Sigma, Phi, and Tau for each IMT
         for iimt in range(nimt):
