@@ -8,20 +8,26 @@ from importlib import import_module
 # third party imports
 import numpy as np
 from openquake.hazardlib import const
-from openquake.hazardlib.contexts import RuptureContext
 from openquake.hazardlib.gsim.base import GMPE
 from openquake.hazardlib.gsim.boore_2014 import BooreEtAl2014
 from openquake.hazardlib.gsim.campbell_bozorgnia_2014 import CampbellBozorgnia2014
 from openquake.hazardlib.imt import PGA, PGV, SA
 from openquake.hazardlib.valid import gsim
 
-from esi_shakelib.conversions.imc.boore_kishida_2017 import BooreKishida2017
-
 # local imports
+from esi_shakelib.utils.contexts import RuptureContext
+from esi_shakelib.utils.gmpe_coeffs import get_gmpe_sa_periods
+from esi_shakelib.conversions.imc.boore_kishida_2017 import BooreKishida2017
 from esi_shakelib.conversions.imt.abrahamson_bhasin_2020 import AbrahamsonBhasin2020
-from esi_shakelib.gmpe.nga_east import NGAEast  # Special case GMPEs
 from esi_shakelib.multiutils import gmpe_gmas
-from esi_shakelib.sites import Sites
+from esi_shakelib.sites import addDepthParameters
+
+# These are imported for their side effects...
+import esi_shakelib.gmpe.atkinson_macias_2009
+import esi_shakelib.gmpe.zhao_2006
+import esi_shakelib.gmpe.kuehn_2020
+import esi_shakelib.gmpe.ak_bias_gmms
+import esi_shakelib.gmpe.nga_east
 
 
 def set_sites_depth_parameters(sites, gmpe):
@@ -44,18 +50,51 @@ def set_sites_depth_parameters(sites, gmpe):
     if gmpe == "[MultiGMPE]":
         return sites
 
-    Sites._addDepthParameters(sites)
+    addDepthParameters(sites)
 
-    if (
-        gmpe == "[AbrahamsonEtAl2014]"
-        or gmpe == '[AbrahamsonEtAl2014]\nregion = "TWN"'
-        or gmpe == '[AbrahamsonEtAl2014]\nregion = "CHN"'
+    if gmpe in (
+        "[AbrahamsonEtAl2014]",
+        '[AbrahamsonEtAl2014]\nregion = "TWN"',
+        '[AbrahamsonEtAl2014]\nregion = "CHN"',
     ):
         sites.z1pt0 = sites.z1pt0_ask14_cal
     if gmpe == '[AbrahamsonEtAl2014]\nregion = "JPN"':
         sites.z1pt0 = sites.z1pt0_ask14_jpn
     if gmpe == "[ChiouYoungs2014]" or isinstance(gmpe, BooreEtAl2014):
         sites.z1pt0 = sites.z1pt0_cy14_cal
+
+    gmpe_name = f"{gmpe}"
+    if "KuehnEtAl202" in gmpe_name:
+        if "Japan" in gmpe_name:
+            sites.z2pt5 = sites.z2pt5_cb14_jpn
+        else:
+            sites.z2pt5 = sites.z2pt5_cb14_cal
+    if "AbrahamsonGulerce2020S" in gmpe_name:
+        if "Japan" in gmpe_name:
+            sites.z2pt5 = sites.z2pt5_cb14_jpn
+        else:
+            sites.z2pt5 = sites.z2pt5_cb14_cal
+    if "ZhaoEtAl2006S" in gmpe_name:
+        if "Japan" in gmpe_name:
+            sites.z2pt5 = sites.z2pt5_cb14_jpn
+        else:
+            sites.z2pt5 = sites.z2pt5_cb14_cal
+    if "AtkinsonMacias2009" in gmpe_name:
+        if "Japan" in gmpe_name:
+            sites.z2pt5 = sites.z2pt5_cb14_jpn
+        else:
+            sites.z2pt5 = sites.z2pt5_cb14_cal
+    if "KuehnEtAl2020S" in gmpe_name:
+        # This is just a WAG at what this GMPE wants
+        if "Japan" in gmpe_name:
+            sites.z2pt5 = sites.z2pt5_cb14_jpn
+        else:
+            sites.z2pt5 = sites.z2pt5_cb14_cal
+        sites.z1pt0 = sites.z1pt0_cy14_cal
+
+    if "KothaEtAl2020ESHM20" in gmpe_name:
+        sites.region = np.zeros_like(sites.vs30)
+
     if isinstance(gmpe, CampbellBozorgnia2014):
         if (
             gmpe == "[CampbellBozorgnia2014JapanSite]"
@@ -95,17 +134,49 @@ def stuff_context(sites, rup, dists):
         RuptureContext: A new RuptureContext whose attributes are all of
         the elements of the three inputs.
     """
+    if (
+        isinstance(sites, np.recarray)
+        and isinstance(rup, np.recarray)
+        and isinstance(dists, np.recarray)
+        and np.all(sites == rup)
+        and np.all(rup == dists)
+    ):
+        # We're already a rec array
+        return rup
+
+    shape = None
     ctx = RuptureContext()
+    for name, val in vars(sites).items():
+        if name.startswith("__"):
+            continue
+        # We don't want the sites lons and lats, we want the dists ones
+        if name in ["lons", "lats"]:
+            continue
+        if shape is None:
+            shape = val.shape
+        setattr(ctx, name, val)
+    for name, val in vars(rup).items():
+        if name.startswith("__"):
+            continue
+        if not isinstance(val, np.ndarray):
+            val = np.full(shape, val)
+        setattr(ctx, name, val)
+    for name, val in vars(dists).items():
+        if name.startswith("__"):
+            continue
+        setattr(ctx, name, val)
 
-    for name in [name for name in vars(sites) if not name.startswith("__")]:
-        setattr(ctx, name, getattr(sites, name))
-    for name in [name for name in vars(rup) if not name.startswith("__")]:
-        setattr(ctx, name, getattr(rup, name))
-    for name in [name for name in vars(dists) if not name.startswith("__")]:
-        setattr(ctx, name, getattr(dists, name))
+    ctx.occurrence_rate = np.full(shape, 1e-5)
 
-    ctx.occurrence_rate = 1e-5
-    return ctx
+    cols = []
+    arrs = []
+    for key, val in vars(ctx).items():
+        if key.startswith("__"):
+            continue
+        cols.append(key)
+        arrs.append(val)
+    cx = np.rec.fromarrays(arrs, names=cols)
+    return cx
 
 
 def get_gmpe_from_name(name, conf):
@@ -114,7 +185,7 @@ def get_gmpe_from_name(name, conf):
     # We'll want to import any other GMPEs we add at the top of this module
     # so that gsim() picks them up; anything in OQ is already included
     if name == "NullGMPE":
-        mod = import_module(conf["gmpe_modules"][name][1])
+        _ = import_module(conf["gmpe_modules"][name][1])
     return gsim(name)
 
 
@@ -138,86 +209,28 @@ class MultiGMPE(GMPE):
 
         """  # noqa
 
-        # ---------------------------------------------------------------------
-        # Sort out shapes of the sites and dists elements
-        # Need to turn all 2D arrays into 1D arrays because of
-        # inconsistencies in how arrays are handled in OpenQuake.
-        # ---------------------------------------------------------------------
-        # shapes = []
-        # for k, v in sites.__dict__.items():
-        #     if k == "_slots_":
-        #         continue
-        #     if (k != "lons") and (k != "lats"):
-        #         shapes.append(v.shape)
-        #         sites.__dict__[k] = np.reshape(sites.__dict__[k], (-1,))
-        # for k, v in dists.__dict__.items():
-        #     if k == "_slots_":
-        #         continue
-        #     if (k != "lons") and (k != "lats") and v is not None:
-        #         shapes.append(v.shape)
-        #         dists.__dict__[k] = np.reshape(dists.__dict__[k], (-1,))
-        # shapeset = set(shapes)
-        # if len(shapeset) != 1:
-        #     raise Exception("All dists and sites elements must have same shape.")
-        # else:
-        #     orig_shape = list(shapeset)[0]
+        for key, val in rup.__dict__.items():
+            if not isinstance(val, np.ndarray):
+                setattr(rup, key, np.full_like(sites.vs30, val))
 
         sd_avail = self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
         if not sd_avail.issuperset(set(stddev_types)):
-            raise Exception("Requested an unavailable stddev_type.")
+            raise TypeError("Requested an unavailable stddev_type.")
 
         if not isinstance(imt, list):
             imt = [imt]
         # Evaluate MultiGMPE:
         lnmu, lnsd = self.__get_mean_and_stddevs__(sites, rup, dists, imt, stddev_types)
 
-        # Check for large-distance cutoff/weights
-        if hasattr(self, "CUTOFF_DISTANCE"):
-            lnmu_large, lnsd_large = self.__get_mean_and_stddevs__(
-                sites, rup, dists, imt, stddev_types, large_dist=True
-            )
-            # Stomp on lnmu and lnsd at large distances
-            dist_cutoff = self.CUTOFF_DISTANCE
-            for ix, iimt in enumerate(imt):
-                lnmu[ix][dists.rjb > dist_cutoff] = lnmu_large[ix][
-                    dists.rjb > dist_cutoff
-                ]
-            for i, this_lnsd in enumerate(lnsd):
-                this_lnsd[dists.rjb > dist_cutoff] = lnsd_large[i][
-                    dists.rjb > dist_cutoff
-                ]
-
-        # Undo reshapes of inputs
-        # for k, v in dists.__dict__.items():
-        #     if k == "_slots_":
-        #         continue
-        #     if (k != "lons") and (k != "lats") and v is not None:
-        #         dists.__dict__[k] = np.reshape(dists.__dict__[k], orig_shape)
-        # for k, v in sites.__dict__.items():
-        #     if k == "_slots_":
-        #         continue
-        #     if (k != "lons") and (k != "lats"):
-        #         sites.__dict__[k] = np.reshape(sites.__dict__[k], orig_shape)
-
-        # Reshape output
-        # lnmu = np.reshape(lnmu, orig_shape)
-        # for i, this_lnsd in enumerate(lnsd):
-        #     this_lnsd = np.reshape(this_lnsd, orig_shape)
-
         return lnmu, lnsd
 
-    def __get_mean_and_stddevs__(
-        self, sites, rup, dists, imt, stddev_types, large_dist=False
-    ):
+    def __get_mean_and_stddevs__(self, sites, rup, dists, imt, stddev_types):
 
         nsd = len(stddev_types)
         # ---------------------------------------------------------------------
         # Sort out which set of weights to use
         # ---------------------------------------------------------------------
-        if large_dist is False:
-            wts = self.WEIGHTS
-        else:
-            wts = self.WEIGHTS_LARGE_DISTANCE
+        wts = self.WEIGHTS
 
         nimt = len(imt)
         # ---------------------------------------------------------------------
@@ -246,17 +259,15 @@ class MultiGMPE(GMPE):
                 imt.__name__ for imt in list(gmpe.DEFINED_FOR_INTENSITY_MEASURE_TYPES)
             ]
 
+            timt = copy.copy(imt)
             if (
                 not isinstance(gmpe, MultiGMPE)
-                and PGV() in imt
+                and PGV() in timt
                 and ("PGV" not in gmpe_imts)
             ):
                 ab2020 = AbrahamsonBhasin2020(rup.mag)
-                timt = copy.copy(imt)
                 pgv_ix = timt.index(PGV())
                 timt[pgv_ix] = SA(ab2020.getTref())
-            else:
-                timt = copy.copy(imt)
 
             # -----------------------------------------------------------------
             # Grab GMPE_LIMITS in gmpe instance for later as the multigmpe
@@ -280,7 +291,7 @@ class MultiGMPE(GMPE):
                             vs30min = float(v[0])
                             vs30max = float(v[1])
                             sites.vs30 = np.clip(sites.vs30, vs30min, vs30max)
-                            Sites._addDepthParameters(sites)
+                            addDepthParameters(sites)
 
             # -----------------------------------------------------------------
             # Evaluate
@@ -310,15 +321,6 @@ class MultiGMPE(GMPE):
                     for ix, _ in enumerate(stddev_types):
                         lsd[pgv_ix * nsd + ix] = tsd[ix]
 
-                # -------------------------------------------------------------
-                # -------------------------------------------------------------
-                if self.HAS_SITE[i] is False:
-                    for kk, kimt in enumerate(timt):
-                        lamps = self.__get_site_factors__(
-                            sites, rup, dists, kimt, default=True
-                        )
-                        lmean[kk] = lmean[kk] + lamps
-
                 # -------------------------------------------------------------
                 # Convertions due to component definition
                 # -------------------------------------------------------------
@@ -415,11 +417,11 @@ class MultiGMPE(GMPE):
             MultiGMPE object.
 
         """
-        IMC = getattr(const.IMC, conf["interp"]["component"])
+        imc = getattr(const.IMC, conf["interp"]["component"])
         selected_gmpe = conf["modeling"]["gmpe"]
 
-        logging.debug(f"selected_gmpe: {selected_gmpe}")
-        logging.debug(f"IMC: {IMC}")
+        logging.debug("selected_gmpe: %s", selected_gmpe)
+        logging.debug("IMC: %s", imc)
 
         # ---------------------------------------------------------------------
         # Allow for selected_gmpe to be found in either conf['gmpe_sets'] or
@@ -433,8 +435,8 @@ class MultiGMPE(GMPE):
             gmpe_set_weights = [
                 float(w) for w in conf["gmpe_sets"][selected_gmpe]["weights"]
             ]
-            logging.debug(f"selected_gmpe_sets: {selected_gmpe_sets}")
-            logging.debug(f"gmpe_set_weights: {gmpe_set_weights}")
+            logging.debug("selected_gmpe_sets: %s", selected_gmpe_sets)
+            logging.debug("gmpe_set_weights: %s", gmpe_set_weights)
 
             # -----------------------------------------------------------------
             # If it is a GMPE set, does it contain GMPEs or GMPE sets?
@@ -445,28 +447,41 @@ class MultiGMPE(GMPE):
 
             if set_of_sets is True:
                 mgmpes = []
-                for s in selected_gmpe_sets:
-                    mgmpes.append(
-                        cls.__multigmpe_from_gmpe_set__(conf, s, filter_imt=filter_imt)
+                mweights = []
+                for ix, s in enumerate(selected_gmpe_sets):
+                    mgmpe = cls.__multigmpe_from_gmpe_set__(
+                        conf, s, filter_imt=filter_imt
                     )
-                out = MultiGMPE.__from_list__(mgmpes, gmpe_set_weights, imc=IMC)
+                    if mgmpe is None:
+                        continue
+                    mgmpes.append(mgmpe)
+                    mweights.append(gmpe_set_weights[ix])
+                if len(mgmpes) == 0:
+                    return None
+                mweights = np.array(mweights)
+                mweights = mweights / np.sum(mweights)
+                out = MultiGMPE.__from_list__(mgmpes, mweights, imc=imc)
             elif set_of_gmpes is True:
                 out = cls.__multigmpe_from_gmpe_set__(
                     conf, selected_gmpe, filter_imt=filter_imt
                 )
+                if out is None:
+                    return None
             else:
                 raise TypeError(
-                    "%s must consist exclusively of keys in "
-                    "conf['gmpe_modules'] or conf['gmpe_sets']" % selected_gmpe
+                    f"{selected_gmpe} must consist exclusively of keys in "
+                    "conf['gmpe_modules'] or conf['gmpe_sets']"
                 )
         elif selected_gmpe in conf["gmpe_modules"].keys():
             modinfo = conf["gmpe_modules"][selected_gmpe]
             # mod = import_module(modinfo[1])
             # tmpclass = getattr(mod, modinfo[0])
-            # out = MultiGMPE.__from_list__([tmpclass()], [1.0], imc=IMC)
+            # out = MultiGMPE.__from_list__([tmpclass()], [1.0], imc=imc)
             out = MultiGMPE.__from_list__(
-                [get_gmpe_from_name(modinfo[0], conf)], [1.0], imc=IMC
+                [get_gmpe_from_name(modinfo[0], conf)], [1.0], imc=imc, imt=filter_imt
             )
+            if out is None:
+                return None
         else:
             raise TypeError(
                 "conf['modeling']['gmpe'] must be a key in "
@@ -495,12 +510,13 @@ class MultiGMPE(GMPE):
 
         return out
 
-    def __multigmpe_from_gmpe_set__(conf, set_name, filter_imt=None):
+    @classmethod
+    def __multigmpe_from_gmpe_set__(cls, conf, set_name, filter_imt=None):
         """
         Private method for constructing a MultiGMPE from a set_name.
 
         Args:
-            conf (ConfigObj): A ShakeMap config object.
+            conf (dict): A ShakeMap ConfigObj config object.
             filter_imt (IMT): An optional IMT to filter/reweight the GMPE list.
             set_name (str): Set name; must correspond to a key in
                 conf['set_name'].
@@ -509,50 +525,13 @@ class MultiGMPE(GMPE):
             MultiGMPE.
 
         """
-        IMC = getattr(const.IMC, conf["interp"]["component"])
+        imc = getattr(const.IMC, conf["interp"]["component"])
 
         selected_gmpes = conf["gmpe_sets"][set_name]["gmpes"]
         selected_gmpe_weights = [
             float(w) for w in conf["gmpe_sets"][set_name]["weights"]
         ]
 
-        # Check for large distance GMPEs
-        if "weights_large_dist" in conf["gmpe_sets"][set_name].keys():
-            if not conf["gmpe_sets"][set_name]["weights_large_dist"]:
-                selected_weights_large_dist = None
-            else:
-                selected_weights_large_dist = [
-                    float(w) for w in conf["gmpe_sets"][set_name]["weights_large_dist"]
-                ]
-        else:
-            selected_weights_large_dist = None
-
-        if "dist_cutoff" in conf["gmpe_sets"][set_name].keys():
-            if np.isnan(conf["gmpe_sets"][set_name]["dist_cutoff"]):
-                selected_dist_cutoff = None
-            else:
-                selected_dist_cutoff = float(conf["gmpe_sets"][set_name]["dist_cutoff"])
-        else:
-            selected_dist_cutoff = None
-
-        if "site_gmpes" in conf["gmpe_sets"][set_name].keys():
-            if not conf["gmpe_sets"][set_name]["site_gmpes"]:
-                selected_site_gmpes = None
-            else:
-                selected_site_gmpes = conf["gmpe_sets"][set_name]["site_gmpes"]
-        else:
-            selected_site_gmpes = None
-
-        if "weights_site_gmpes" in conf["gmpe_sets"][set_name].keys():
-            if not conf["gmpe_sets"][set_name]["weights_site_gmpes"]:
-                selected_weights_site_gmpes = None
-            else:
-                selected_weights_site_gmpes = conf["gmpe_sets"][set_name][
-                    "weights_site_gmpes"
-                ]
-        else:
-            selected_weights_site_gmpes = None
-
         # ---------------------------------------------------------------------
         # Import GMPE modules and initialize classes into list
         # ---------------------------------------------------------------------
@@ -576,67 +555,21 @@ class MultiGMPE(GMPE):
         else:
             filtered_gmpes, filtered_wts = gmpes, selected_gmpe_weights
 
-        # ---------------------------------------------------------------------
-        # Import site GMPEs
-        # ---------------------------------------------------------------------
-        if selected_site_gmpes is not None:
-            if isinstance(selected_site_gmpes, str):
-                selected_site_gmpes = [selected_site_gmpes]
-            site_gmpes = []
-            for g in selected_site_gmpes:
-                # This is the old school way of importing the modules; I'm
-                # leaving it in here temporarily just for documentation.
-                # mod = import_module(conf['gmpe_modules'][g][1])
-                # tmpclass = getattr(mod, conf['gmpe_modules'][g][0])
-                # site_gmpes.append(tmpclass())
-                gmpe_name = conf["gmpe_modules"][g][0]
-                site_gmpes.append(get_gmpe_from_name(gmpe_name, conf))
-        else:
-            site_gmpes = None
-
-        # ---------------------------------------------------------------------
-        # Filter out site GMPEs not applicable to this period
-        # ---------------------------------------------------------------------
-        if site_gmpes is not None:
-            if filter_imt is not None:
-                filtered_site_gmpes, filtered_site_wts = filter_gmpe_list(
-                    site_gmpes, selected_weights_site_gmpes, filter_imt
-                )
-            else:
-                filtered_site_gmpes = copy.copy(site_gmpes)
-                filtered_site_wts = copy.copy(selected_weights_site_gmpes)
-        else:
-            filtered_site_gmpes = None
-            filtered_site_wts = None
-
         # ---------------------------------------------------------------------
         # Construct MultiGMPE
         # ---------------------------------------------------------------------
-        logging.debug(f"    filtered_gmpes: {filtered_gmpes}")
-        logging.debug(f"    filtered_wts: {filtered_wts}")
+        logging.debug("    filtered_gmpes: %s", filtered_gmpes)
+        logging.debug("    filtered_wts: %s", filtered_wts)
+
+        if len(filtered_gmpes) == 0:
+            return None
 
         mgmpe = MultiGMPE.__from_list__(
             filtered_gmpes,
             filtered_wts,
-            default_gmpes_for_site=filtered_site_gmpes,
-            default_gmpes_for_site_weights=filtered_site_wts,
-            imc=IMC,
+            imc=imc,
         )
 
-        # ---------------------------------------------------------------------
-        # Append large-distance info if specified
-        # ---------------------------------------------------------------------
-        if selected_dist_cutoff is not None:
-            if filter_imt is not None:
-                filtered_gmpes_ld, filtered_wts_ld = filter_gmpe_list(
-                    gmpes, selected_weights_large_dist, filter_imt
-                )
-            else:
-                filtered_wts_ld = copy.copy(selected_weights_large_dist)
-
-            mgmpe.CUTOFF_DISTANCE = copy.copy(selected_dist_cutoff)
-            mgmpe.WEIGHTS_LARGE_DISTANCE = copy.copy(filtered_wts_ld)
-
         mgmpe.DESCRIPTION = set_name
 
         return mgmpe
@@ -647,8 +580,7 @@ class MultiGMPE(GMPE):
         gmpes,
         weights,
         imc=const.IMC.GREATER_OF_TWO_HORIZONTAL,
-        default_gmpes_for_site=None,
-        default_gmpes_for_site_weights=None,
+        imt=None,
         reference_vs30=760,
     ):
         """
@@ -664,27 +596,11 @@ class MultiGMPE(GMPE):
             imc: Requested intensity measure component. Must be one listed
                 `here <http://docs.openquake.org/oq-hazardlib/master/const.html?highlight=imc#openquake.hazardlib.const.IMC>`__.
                 The amplitudes returned by the GMPEs will be converted to this
-                IMT. Default is 'GREATER_OF_TWO_HORIZONTAL', which is used by
+                IMC. Default is 'GREATER_OF_TWO_HORIZONTAL', which is used by
                 ShakeMap. See discussion in
                 `this section <http://usgs.github.io/shakemap/tg_choice_of_parameters.html#use-of-peak-values-rather-than-mean>`__
                 of the ShakeMap manual.
 
-            default_gmpes_for_site (list):
-                Optional list of OpenQuake GMPE instance to use as a site term
-                for any of the GMPEs that do not have a site term.
-
-                Notes:
-
-                    * We do not check for consistency in the reference rock
-                      defintion, so the user nees to be aware of this issue and
-                      holds responsibiilty for ensuring compatibility.
-                    * We check whether or not a GMPE has a site term by c
-                      hecking the REQUIRES_SITES_PARAMETERS slot for vs30.
-
-            default_gmpes_for_site_weights: Weights for default_gmpes_for_site.
-                Must sum to one and be same length as default_gmpes_for_site.
-                If None, then weights are set to be equal.
-
             reference_vs30:
                 Reference rock Vs30 in m/s. We do not check that this matches
                 the reference rock in the GMPEs so this is the responsibility
@@ -697,14 +613,14 @@ class MultiGMPE(GMPE):
         # ---------------------------------------------------------------------
 
         if np.abs(np.sum(weights) - 1.0) > 1e-7:
-            raise Exception("Weights must sum to one.")
+            raise ValueError("Weights must sum to one.")
 
         # ---------------------------------------------------------------------
         # Check that length of GMPE weights equals length of gmpe list
         # ---------------------------------------------------------------------
 
         if len(weights) != len(gmpes):
-            raise Exception("Length of weights must match length of GMPE list.")
+            raise ValueError("Length of weights must match length of GMPE list.")
 
         # ---------------------------------------------------------------------
         # Check that gmpes is a list of OQ GMPE instances
@@ -712,7 +628,12 @@ class MultiGMPE(GMPE):
 
         for g in gmpes:
             if not isinstance(g, GMPE):
-                raise Exception(f'"{g}" is a {type(g)} not a GMPE instance.')
+                raise ValueError(f'"{g}" is a {type(g)} not a GMPE instance.')
+
+        if imt is not None:
+            gmpes, weights = filter_gmpe_list(gmpes, weights, imt)
+            if len(gmpes) == 0:
+                return None
 
         self = cls()
         self.GMPES = gmpes
@@ -736,7 +657,13 @@ class MultiGMPE(GMPE):
         # defined for PGV, in which case we will convert from PGV to MI,
         # otherwise use PGA or Sa.
         # ---------------------------------------------------------------------
-        haspgv = [PGV in set(g.DEFINED_FOR_INTENSITY_MEASURE_TYPES) for g in gmpes]
+        haspgv = [
+            any(
+                "PGV" in imtype.__name__
+                for imtype in g.DEFINED_FOR_INTENSITY_MEASURE_TYPES
+            )
+            for g in gmpes
+        ]
         self.ALL_GMPES_HAVE_PGV = all(haspgv)
 
         # ---------------------------------------------------------------------
@@ -767,64 +694,6 @@ class MultiGMPE(GMPE):
         # ---------------------------------------------------------------------
         self.HAS_SITE = ["vs30" in g.REQUIRES_SITES_PARAMETERS for g in gmpes]
 
-        # ---------------------------------------------------------------------
-        # Checks and sort out defaults
-        # ---------------------------------------------------------------------
-
-        # things to check if default_gmpes_for_site is provided
-        if default_gmpes_for_site is not None:
-            # check that default_gmpe_for_site are OQ GMPEs or None
-            for g in default_gmpes_for_site:
-                if not isinstance(g, GMPE):
-                    raise Exception(f'"{g}" is not a GMPE instance.')
-
-            # apply default weights if necessary
-            if default_gmpes_for_site_weights is None:
-                n = len(default_gmpes_for_site)
-                default_gmpes_for_site_weights = [1 / n] * n
-
-        # Things to check if one or more GMPE does not have a site term
-        if not all(self.HAS_SITE):
-            # Raise an exception if no default site is provided
-            if default_gmpes_for_site is None:
-                raise Exception(
-                    "Must provide default_gmpes_for_site if one or"
-                    " more GMPE does not have site term."
-                )
-
-            # If weights are unspecified, use equal weight
-            if default_gmpes_for_site_weights is None:
-                default_gmpes_for_site_weights = [
-                    1 / len(default_gmpes_for_site)
-                ] * len(default_gmpes_for_site)
-
-            # check that length of default_gmpe_for_site matches length of
-            # default_gmpe_for_site_weights
-            if len(default_gmpes_for_site_weights) != len(default_gmpes_for_site):
-                raise Exception(
-                    "Length of default_gmpes_for_site_weights "
-                    "must match length of default_gmpes_for_site "
-                    "list."
-                )
-
-            # check weights sum to one if needed
-            if not all(self.HAS_SITE):
-                if np.sum(default_gmpes_for_site_weights) != 1.0:
-                    raise Exception(
-                        "default_gmpes_for_site_weights must sum" " to one."
-                    )
-
-        # Note: if ALL of the GMPEs do not have a site term (requiring Vs30),
-        #       then REQUIRES_SITES_PARAMETERS for the MultiGMPE will not
-        #       include Vs30 even though it will be needed to compute the
-        #       default site term. So if the site checks have passed to this
-        #       point, we should add Vs30 to the set of required site pars:
-        self.REQUIRES_SITES_PARAMETERS = set.union(
-            set(self.REQUIRES_SITES_PARAMETERS), set(["vs30"])
-        )
-
-        self.DEFAULT_GMPES_FOR_SITE = default_gmpes_for_site
-        self.DEFAULT_GMPES_FOR_SITE_WEIGHTS = default_gmpes_for_site_weights
         self.REFERENCE_VS30 = reference_vs30
 
         # ---------------------------------------------------------------------
@@ -841,64 +710,6 @@ class MultiGMPE(GMPE):
 
         return self
 
-    def __get_site_factors__(self, sites, rup, dists, imt, default=False):
-        """
-        Method for computing site amplification factors from the defalut GMPE
-        to be applied to GMPEs which do not have a site term.
-
-        **NOTE** Amps are calculated in natural log units and so the ln(amp)
-        is returned.
-
-        Args:
-            sites (SitesContext): Instance of SitesContext.
-            rup (RuptureContext): Instance of RuptureContext.
-            dists (DistancesContext): Instance of DistancesContext.
-            imt: An instance openquake.hazardlib.imt.
-            default (bool): Boolean of whether or not to return the
-                amplificaiton factors for the gmpes or default_gmpes_for_site.
-                This argument is primarily only intended to be used internally
-                for when we just need to access the default amplifications to
-                apply to those GMPEs that do not have site terms.
-
-        Returns:
-            Site amplifications in natural log units.
-        """
-
-        # ---------------------------------------------------------------------
-        # Make reference sites context
-        # ---------------------------------------------------------------------
-
-        ref_sites = copy.deepcopy(sites)
-        ref_sites.vs30 = np.full_like(sites.vs30, self.REFERENCE_VS30)
-        # TODO: Should we reset the Sites depth parameters here? Probably.
-
-        # ---------------------------------------------------------------------
-        # If default True, construct new MultiGMPE with default GMPE/weights
-        # ---------------------------------------------------------------------
-        if default is True:
-            tmp = MultiGMPE.__from_list__(
-                self.DEFAULT_GMPES_FOR_SITE,
-                self.DEFAULT_GMPES_FOR_SITE_WEIGHTS,
-                self.DEFINED_FOR_INTENSITY_MEASURE_COMPONENT,
-            )
-
-        # ---------------------------------------------------------------------
-        # If default False, just use self
-        # ---------------------------------------------------------------------
-        else:
-            tmp = self
-
-        lmean, lsd = tmp.get_mean_and_stddevs(
-            sites, rup, dists, imt, list(tmp.DEFINED_FOR_STANDARD_DEVIATION_TYPES)
-        )
-        lmean_ref, lsd = tmp.get_mean_and_stddevs(
-            ref_sites, rup, dists, imt, list(tmp.DEFINED_FOR_STANDARD_DEVIATION_TYPES)
-        )
-
-        lamps = lmean - lmean_ref
-
-        return lamps
-
     def __describe__(self):
         """
         Construct a dictionary that describes the MultiGMPE.
@@ -943,19 +754,26 @@ def filter_gmpe_list(gmpes, wts, imt):
     per_max = [np.max(get_gmpe_sa_periods(g)) for g in gmpes]
     per_min = [np.min(get_gmpe_sa_periods(g)) for g in gmpes]
     if imt == PGA():
-        sgmpe = [g for g in gmpes if PGA in g.DEFINED_FOR_INTENSITY_MEASURE_TYPES]
+        sgmpe = [
+            g
+            for g in gmpes
+            if any("PGA" in t.__name__ for t in g.DEFINED_FOR_INTENSITY_MEASURE_TYPES)
+        ]
         swts = [
             w
             for g, w in zip(gmpes, wts)
-            if PGA in g.DEFINED_FOR_INTENSITY_MEASURE_TYPES
+            if any("PGA" in t.__name__ for t in g.DEFINED_FOR_INTENSITY_MEASURE_TYPES)
         ]
     elif imt == PGV():
         sgmpe = []
         swts = []
         for i, this_gmpe in enumerate(gmpes):
-            if (PGV in this_gmpe.DEFINED_FOR_INTENSITY_MEASURE_TYPES) or (
-                per_max[i] >= 1.0 and per_min[i] <= 1.0
-            ):
+            if (
+                any(
+                    "PGV" in t.__name__
+                    for t in this_gmpe.DEFINED_FOR_INTENSITY_MEASURE_TYPES
+                )
+            ) or (per_max[i] >= 1.0 >= per_min[i]):
                 sgmpe.append(this_gmpe)
                 swts.append(wts[i])
     else:
@@ -963,64 +781,16 @@ def filter_gmpe_list(gmpes, wts, imt):
         sgmpe = []
         swts = []
         for i, this_gmpe in enumerate(gmpes):
-            if per_max[i] >= per and per_min[i] <= per:
+            if per_max[i] >= per >= per_min[i]:
                 sgmpe.append(this_gmpe)
                 swts.append(wts[i])
 
     if len(sgmpe) == 0:
-        raise KeyError(f"No applicable GMPEs from GMPE list for {str(imt)}")
+        return [], []
+    #    raise KeyError(f"No applicable GMPEs from GMPE list for {str(imt)}")
 
     # Scale weights to sum to one
     swts = np.array(swts)
     swts = swts / np.sum(swts)
 
     return sgmpe, swts
-
-
-def get_gmpe_sa_periods(gmpe):
-    """
-    Method to extract the SA periods defined by a GMPE.
-
-    Args:
-        gmpe (GMPE): A GMPE instance.
-
-    Retunrs:
-        list: List of periods.
-
-    """
-    if gmpe == "[NGAEast]":
-        per = gmpe.per_array
-    else:
-        ctab = get_gmpe_coef_table(gmpe).sa_coeffs
-        ilist = list(ctab.keys())
-        per = [i.period for i in ilist]
-    return per
-
-
-def get_gmpe_coef_table(gmpe):
-    """
-    Method for finding the (or "a") GMPE table.
-
-    Notes:
-
-      *  The reason for the complexity here is that there can be multiple
-         coefficient tables, and some of them may not have the sa_coeffs
-         attribute, which is the main reason for getting the table.
-      *  We are also assuming that if there are more than one  coefficient
-         table, the range of periods will be the same across all of the
-         tables.
-
-    Args:
-        gmpe (GMPE): An OQ GMPE instance.
-
-    Returns:
-        The associated coefficient table.
-
-    """
-    stuff = gmpe.__dir__()
-    coef_list = [s for s in stuff if "COEFFS" in s]
-    for coef_sel in coef_list:
-        cobj = getattr(gmpe, coef_sel)
-        if "sa_coeffs" in cobj.__dir__():
-            return cobj
-    raise Exception(f"GMPE {gmpe} does not contain sa_coeffs attribute.")
